@@ -32,8 +32,10 @@ pub struct SyncChain {
 impl ChainInstance for SyncChain {
     async fn append_delete(&self, txn_id: TxnId, path: TCPathBuf, key: Value) -> TCResult<()> {
         let mut block = self.history.write_latest(txn_id).await?;
+
         block.clear_until(&txn_id);
         block.append_delete(txn_id, path, key);
+
         Ok(())
     }
 
@@ -46,6 +48,7 @@ impl ChainInstance for SyncChain {
     ) -> TCResult<()> {
         {
             let mut block = self.history.write_latest(*txn.id()).await?;
+
             block.clear_until(txn.id());
         }
 
@@ -65,6 +68,7 @@ impl ChainInstance for SyncChain {
         self.subject.restore(txn, subject).await?;
 
         let mut block = self.history.write_latest(*txn.id()).await?;
+
         *block = ChainBlock::with_txn(NULL_HASH, *txn.id());
 
         Ok(())
@@ -86,7 +90,7 @@ impl Persist<fs::Dir> for SyncChain {
     }
 
     async fn load(txn: &Txn, schema: Self::Schema, dir: fs::Dir) -> TCResult<Self> {
-        let is_new = dir.is_empty(txn.id()).await?;
+        let is_new = dir.is_empty(*txn.id()).await?;
 
         let subject = Subject::load(txn, schema.clone(), &dir).await?;
 
@@ -96,7 +100,7 @@ impl Persist<fs::Dir> for SyncChain {
             History::load(txn, (), dir).await?
         };
 
-        let latest = history.latest_block_id(txn.id()).await?;
+        let latest = history.latest_block_id(*txn.id()).await?;
         if latest > 0 {
             return Err(TCError::internal(format!(
                 "a SyncChain can only have one block, found {}",
