@@ -78,9 +78,9 @@ impl<'a> Handler<'a> for ConstantHandler {
     }
 }
 
-struct CopyHandler;
+struct CopyFromHandler;
 
-impl<'a> Handler<'a> for CopyHandler {
+impl<'a> Handler<'a> for CopyFromHandler {
     fn post<'b>(self: Box<Self>) -> Option<PostHandler<'a, 'b>>
     where
         'b: 'a,
@@ -144,7 +144,8 @@ impl<'a> Handler<'a> for CopyDenseHandler {
                 let file = create_file(txn).await?;
                 DenseTensorFile::from_values(file, txn_id, shape, dtype, elements)
                     .map_ok(DenseTensor::from)
-                    .map_ok(Collection::from)
+                    .map_ok(Tensor::from)
+                    .map_ok(Collection::Tensor)
                     .map_ok(State::Collection)
                     .await
             })
@@ -864,7 +865,7 @@ impl Route for Static {
         match path[0].as_str() {
             "dense" => TensorType::Dense.route(&path[1..]),
             "sparse" => TensorType::Sparse.route(&path[1..]),
-            "copy_from" if path.len() == 1 => Some(Box::new(CopyHandler)),
+            "copy_from" if path.len() == 1 => Some(Box::new(CopyFromHandler)),
             "einsum" if path.len() == 1 => Some(Box::new(EinsumHandler)),
             _ => None,
         }
