@@ -7,24 +7,24 @@ use futures::stream::{FuturesUnordered, StreamExt};
 use log::debug;
 
 use tc_error::*;
-use tcgeneric::{Id, Instance, Map};
+use tcgeneric::{Id, Instance, Map, Tuple};
 
 use crate::route::Public;
 use crate::scalar::{Refer, Scope};
-use crate::state::State;
+use crate::state::{State, ToState};
 use crate::txn::Txn;
 
 /// An `OpDef` executor.
 pub struct Executor<'a, T> {
-    txn: Txn,
+    txn: &'a Txn,
     scope: Scope<'a, T>,
 }
 
-impl<'a, T: Instance + Public> Executor<'a, T> {
+impl<'a, T: ToState + Instance + Public> Executor<'a, T> {
     /// Construct a new `Executor` with the given [`Txn`] context and initial state.
     pub fn new<S: Into<State>, I: IntoIterator<Item = (Id, S)>>(
-        txn: Txn,
-        subject: &'a T,
+        txn: &'a Txn,
+        subject: Option<&'a T>,
         data: I,
     ) -> Self {
         let scope = Scope::new(subject, data);
@@ -32,8 +32,8 @@ impl<'a, T: Instance + Public> Executor<'a, T> {
     }
 
     pub fn with_context<S: Into<State>, I: IntoIterator<Item = (Id, S)>>(
-        txn: Txn,
-        subject: &'a T,
+        txn: &'a Txn,
+        subject: Option<&'a T>,
         context: Map<State>,
         iter: I,
     ) -> Self {
@@ -41,6 +41,10 @@ impl<'a, T: Instance + Public> Executor<'a, T> {
         Self { txn, scope }
     }
 
+    pub fn from_scope(txn: &'a Txn, scope: Scope<'a, T>) -> Self {
+        Self { txn, scope }
+    }
+
     /// Resolve the state of the variable `capture`, including any of its dependencies.
     pub async fn capture(mut self, capture: Id) -> TCResult<State> {
         debug!("execute op & capture {}", capture);
@@ -58,12 +62,12 @@ impl<'a, T: Instance + Public> Executor<'a, T> {
                     let mut deps = HashSet::new();
                     state.requires(&mut deps);
 
+                    if deps.contains(&id) {
+                        return Err(TCError::bad_request("circular dependency", id));
+                    }
+
                     let mut ready = true;
                     for dep_id in deps.into_iter() {
-                        if dep_id == id {
-                            return Err(TCError::bad_request("circular dependency", id));
-                        }
-
                         if self.scope.resolve_id(&dep_id)?.is_ref() {
                             ready = false;
                             unvisited.push_back(dep_id);
@@ -71,6 +75,7 @@ impl<'a, T: Instance + Public> Executor<'a, T> {
                     }
 
                     if ready {
+                        debug!("all deps resolved for {}", state);
                         pending.push(id);
                     } else {
                         debug!("{} still has unresolved deps", id);
@@ -91,16 +96,19 @@ impl<'a, T: Instance + Public> Executor<'a, T> {
             {
                 let mut providers = FuturesUnordered::new();
                 for id in pending.into_iter() {
-                    let state = self.scope.resolve_id(&id)?.clone();
+                    debug!("enqueue {}", id);
+                    let state = self.scope.resolve_id(&id)?;
+                    debug!("provider for {} is {}", id, state);
                     providers.push(state.resolve(&self.scope, &self.txn).map(|r| (id, r)));
                 }
 
                 while let Some((id, r)) = providers.next().await {
                     match r {
                         Ok(state) => {
+                            debug!("{} resolved to {}", id, state);
                             resolved.insert(id, state);
                         }
-                        Err(cause) => return Err(cause.consume(format!("error resolving {}", id))),
+                        Err(cause) => return Err(cause.consume(format!("while resolving {}", id))),
                     }
                 }
             }
@@ -108,9 +116,13 @@ impl<'a, T: Instance + Public> Executor<'a, T> {
             self.scope.extend(resolved);
         }
 
-        self.scope
-            .into_inner()
-            .remove(&capture)
-            .ok_or_else(|| TCError::not_found(capture))
+        self.scope.remove(&capture).ok_or_else(|| {
+            let msg = format!(
+                "captured state {} in context {}",
+                capture,
+                self.scope.keys().collect::<Tuple<&Id>>()
+            );
+            TCError::not_found(msg)
+        })
     }
 }
