@@ -18,7 +18,7 @@ use tc_transact::Transaction;
 use tc_value::{Link, Number, TCString, Value, ValueType};
 use tcgeneric::*;
 
-use crate::chain::*;
+use crate::chain::{self, Chain, ChainType, ChainVisitor};
 use crate::closure::*;
 use crate::collection::*;
 use crate::object::{InstanceClass, Object, ObjectType, ObjectVisitor};
@@ -102,37 +102,6 @@ impl NativeClass for StateType {
     }
 }
 
-// impl StateClass for StateType {
-//     type Get = State;
-//
-//     fn try_cast_from_value(self, value: Value) -> TCResult<Self::Get> {
-//         match self {
-//             Self::Chain(ct) => ct.try_cast_from_value(value).map(State::from),
-//             Self::Collection(ct) => ct.try_cast_from_value(value).map(State::from),
-//             Self::Closure => Err(TCError::bad_request("cannot cast into closure from", value)),
-//             Self::Map => {
-//                 let contents: Tuple<Value> = value.try_into()?;
-//                 let contents: Vec<(Id, Value)> = contents
-//                     .try_cast_into(|v| TCError::bad_request("cannot cast into Map from", v))?;
-//
-//                 Ok(State::Map(
-//                     contents
-//                         .into_iter()
-//                         .map(|(id, value)| (id, State::from(value)))
-//                         .collect(),
-//                 ))
-//             }
-//             Self::Object(ot) => ot.try_cast_from_value(value).map(State::Object),
-//             Self::Scalar(st) => st.try_cast_from_value(value).map(State::Scalar),
-//             Self::Stream => Err(TCError::bad_request("cannot cast into Stream from", value)),
-//             Self::Tuple => {
-//                 let contents: Tuple<Value> = value.try_into()?;
-//                 Ok(State::Tuple(contents.into_iter().collect()))
-//             }
-//         }
-//     }
-// }
-
 impl From<BTreeType> for StateType {
     fn from(btt: BTreeType) -> Self {
         CollectionType::BTree(btt).into()
@@ -264,6 +233,7 @@ impl State {
                 .into_iter()
                 .map(|(id, scalar)| (id, State::Scalar(scalar)))
                 .collect()),
+
             other => Err((err)(other)),
         }
     }
@@ -560,6 +530,12 @@ impl From<Tuple<Scalar>> for State {
     }
 }
 
+impl From<Tuple<chain::Subject>> for State {
+    fn from(tuple: Tuple<chain::Subject>) -> Self {
+        Self::Tuple(tuple.into_iter().map(Self::from).collect())
+    }
+}
+
 impl From<Tuple<Value>> for State {
     fn from(tuple: Tuple<Value>) -> Self {
         Self::Scalar(tuple.into())
